From 608a15a5c426261619aac4e6401d4f4e4900cf13 Mon Sep 17 00:00:00 2001
From: Simon lang <simon.lang@outlook.com>
Date: Sun, 13 Jun 2021 15:59:20 +0100
Subject: [PATCH] icomplete-vertical functions to go to first or last entry

These allow to go to the first or last completion entry
when icomplete-scroll is non-nil in fido-vertical-mode,
i.e. when cycling is deactivated
* lisp/icomplete.el (icomplete-vertical-goto-first,
icomplete-vertical-goto-last):
---
 lisp/icomplete.el | 26 ++++++++++++++++++++++++++
 1 file changed, 26 insertions(+)

diff --git a/lisp/icomplete.el b/lisp/icomplete.el
index 08b4ef2030..5cc82de359 100644
--- a/lisp/icomplete.el
+++ b/lisp/icomplete.el
@@ -276,6 +276,30 @@ Last entry becomes the first and can be selected with
            (setcdr last-but-one (cdr (cdr last-but-one)))))
     (completion--cache-all-sorted-completions beg end comps)))
 
+(defun icomplete-vertical-goto-first ()
+  "Go to first completions entry when `icomplete-scroll' is non-nil"
+  (interactive)
+  (let* ((beg (icomplete--field-beg))
+         (end (icomplete--field-end))
+         (comps (completion-all-sorted-completions beg end)))
+    (when icomplete-scroll
+      (while icomplete--scrolled-past
+        (push (pop icomplete--scrolled-past) comps)
+        (setq icomplete--scrolled-completions comps))
+      (completion--cache-all-sorted-completions beg end comps))))
+
+(defun icomplete-vertical-goto-last ()
+  "Go to last completions entry when `icomplete-scroll' is non-nil"
+  (interactive)
+  (let* ((beg (icomplete--field-beg))
+         (end (icomplete--field-end))
+         (comps (completion-all-sorted-completions beg end)))
+    (when icomplete-scroll
+      (while (consp (cdr comps))
+	(push (pop comps) icomplete--scrolled-past)
+	(setq icomplete--scrolled-completions comps)
+	(completion--cache-all-sorted-completions beg end comps)))))
+
 ;;; Helpers for `fido-mode' (or `ido-mode' emulation)
 ;;;
 (defun icomplete-fido-kill ()
@@ -600,6 +624,8 @@ Usually run by inclusion in `minibuffer-setup-hook'."
   (let ((map (make-sparse-keymap)))
     (define-key map (kbd "C-n") 'icomplete-forward-completions)
     (define-key map (kbd "C-p") 'icomplete-backward-completions)
+    (define-key map (kbd "C-M-<") 'icomplete-vertical-goto-first)
+    (define-key map (kbd "C-M->") 'icomplete-vertical-goto-last)
     map)
   "Keymap used by `icomplete-vertical-mode' in the minibuffer.")
 
-- 
2.30.1 (Apple Git-130)

